@@ -99,18 +99,30 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
     }
     else if (AlphabetTools::isCodonAlphabet(alphabet))
     {
-      if (alphabetCode_ & CODON)
-        pFS.reset(new FixedCodonFrequenciesSet(dynamic_cast<const CodonAlphabet*>(alphabet)));
-      else
+      if (alphabetCode_ & CODON) {
+        if (!geneticCode_)
+          throw Exception("BppOFrequenciesSetFormat::read(). No genetic code specified! Consider using 'setGeneticCode'.");
+        pFS.reset(new FixedCodonFrequenciesSet(geneticCode_));
+      } else {
         throw Exception("Codon alphabet not supported.");
+      }
     }
     else
     {
-      pFS.reset(new FixedFrequenciesSet(alphabet, alphabet->getSize()));
+      pFS.reset(new FixedFrequenciesSet(new CanonicalStateMap(alphabet, false)));
     }
   }
   else if (freqName == "Full")
   {
+    unsigned short method = 1;
+    if (args.find("method") != args.end()){
+      if (args["method"] == "local")
+        method=2;
+      else
+        if (args["method"] == "binary")
+          method=3;
+    }
+      
     if (AlphabetTools::isNucleicAlphabet(alphabet))
     {
       if (alphabetCode_ & NUCLEOTIDE)
@@ -121,20 +133,24 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
     else if (AlphabetTools::isProteicAlphabet(alphabet))
     {
       if (alphabetCode_ & PROTEIN)
-        pFS.reset(new FullProteinFrequenciesSet(dynamic_cast<const ProteicAlphabet*>(alphabet)));
+        pFS.reset(new FullProteinFrequenciesSet(dynamic_cast<const ProteicAlphabet*>(alphabet), false, method));
       else
         throw Exception("Protein alphabet not supported.");
     }
     else if (AlphabetTools::isCodonAlphabet(alphabet))
     {
-      if (alphabetCode_ & CODON)
-        pFS.reset(new FullCodonFrequenciesSet(dynamic_cast<const CodonAlphabet*>(alphabet)));
-      else
+      if (alphabetCode_ & CODON) {
+        if (!geneticCode_)
+          throw Exception("BppOFrequenciesSetFormat::read(). No genetic code specified! Consider using 'setGeneticCode'.");
+        pFS.reset(new FullCodonFrequenciesSet(geneticCode_));
+      } else {
         throw Exception("Codon alphabet not supported.");
+      }
     }
     else
     {
-      pFS.reset(new FullFrequenciesSet(alphabet));
+      //NB: jdutheil 25/09/14 => gap models will not be supported before we add the appropriate option!
+      pFS.reset(new FullFrequenciesSet(new CanonicalStateMap(alphabet, false), false, method));
     }
   }
   else if (freqName == "GC")
@@ -150,7 +166,7 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
   // INDEPENDENTWORD
   else if (freqName == "Word")
   {
-    if (!(alphabetCode_ & WORD))
+    if (!(alphabetCode_& WORD))
       throw Exception("Word alphabet not supported.");
     if (!AlphabetTools::isWordAlphabet(alphabet))
       throw Exception("BppOFrequenciesSetFormat::read(...).\n\t Bad alphabet type "
@@ -169,7 +185,7 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
         st += TextTools::toString(i + 1);
       }
 
-      BppOFrequenciesSetFormat nestedReader(alphabetCode_, false);
+      BppOFrequenciesSetFormat nestedReader(alphabetCode_, false, warningLevel_);
       auto_ptr<FrequenciesSet> pFS2(nestedReader.read(pWA->getNAlphabet(0), sAFS, data, false));
       map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
       for (map<string, string>::iterator it = unparsedParameterValuesNested.begin(); it != unparsedParameterValuesNested.end(); it++)
@@ -194,9 +210,9 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
       if (v_sAFS.size() != pWA->getLength())
         throw Exception("BppOFrequenciesSetFormat::read. Number of frequencies (" + TextTools::toString(v_sAFS.size()) + ") does not match length of the words (" + TextTools::toString(pWA->getLength()) + ")");
 
-      for (unsigned i = 0; i < v_sAFS.size(); i++)
+      for (size_t i = 0; i < v_sAFS.size(); ++i)
       {
-        BppOFrequenciesSetFormat nestedReader(alphabetCode_, false);
+        BppOFrequenciesSetFormat nestedReader(alphabetCode_, false, warningLevel_);
         pFS.reset(nestedReader.read(pWA->getNAlphabet(i), v_sAFS[i], data, false));
         map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
         for (map<string, string>::iterator it = unparsedParameterValuesNested.begin(); it != unparsedParameterValuesNested.end(); it++)
@@ -217,6 +233,8 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
     if (!AlphabetTools::isCodonAlphabet(alphabet))
       throw Exception("BppOFrequenciesSetFormat::read.\n\t Bad alphabet type "
                       + alphabet->getAlphabetType() + " for frequenciesset " + freqName + ".");
+    if (!geneticCode_)
+      throw Exception("BppOFrequenciesSetFormat::read(). No genetic code specified! Consider using 'setGeneticCode'.");
 
     const CodonAlphabet* pWA = dynamic_cast<const CodonAlphabet*>(alphabet);
 
@@ -224,7 +242,7 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
     {
       string sAFS = args["frequency"];
 
-      BppOFrequenciesSetFormat nestedReader(alphabetCode_, false);
+      BppOFrequenciesSetFormat nestedReader(alphabetCode_, false, warningLevel_);
       auto_ptr<FrequenciesSet> pFS2(nestedReader.read(pWA->getNAlphabet(0), sAFS, data, false));
       map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
 
@@ -232,7 +250,8 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
       {
         unparsedArguments_["123_" + it->first] = it->second;
       }
-      pFS.reset(new CodonFromUniqueFrequenciesSet(pWA, pFS2.release(), "Codon"));
+      
+      pFS.reset(new CodonFromUniqueFrequenciesSet(geneticCode_, pFS2.release(), "Codon"));
     }
     else
     {
@@ -250,9 +269,9 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
       if (v_sAFS.size() != 3)
         throw Exception("BppOFrequenciesSetFormat::read. Number of frequencies (" + TextTools::toString(v_sAFS.size()) + ") is not three");
 
-      for (unsigned i = 0; i < v_sAFS.size(); i++)
+      for (size_t i = 0; i < v_sAFS.size(); ++i)
       {
-        BppOFrequenciesSetFormat nestedReader(alphabetCode_, false);
+        BppOFrequenciesSetFormat nestedReader(alphabetCode_, false, warningLevel_);
         pFS.reset(nestedReader.read(pWA->getNAlphabet(i), v_sAFS[i], data, false));
         map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
         for (map<string, string>::iterator it = unparsedParameterValuesNested.begin(); it != unparsedParameterValuesNested.end(); it++)
@@ -262,7 +281,9 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
         v_AFS.push_back(pFS.release());
       }
 
-      pFS.reset(new CodonFromIndependentFrequenciesSet(pWA, v_AFS, "Codon"));
+      if (!geneticCode_)
+        throw Exception("BppOFrequenciesSetFormat::read(). No genetic code specified! Consider using 'setGeneticCode'.");
+      pFS.reset(new CodonFromIndependentFrequenciesSet(geneticCode_, v_AFS, "Codon"));
     }
   }
 
@@ -275,34 +296,35 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
       throw Exception("BppOFrequenciesSetFormat::read.\n\t Bad alphabet type "
                       + alphabet->getAlphabetType() + " for frequenciesset " + freqName + ".");
 
-    const CodonAlphabet* pWA = dynamic_cast<const CodonAlphabet*>(alphabet);
+    if (!geneticCode_)
+      throw Exception("BppOFrequenciesSetFormat::read(). No genetic code specified! Consider using 'setGeneticCode'.");
+    
+    const ProteicAlphabet* pPA = dynamic_cast<const ProteicAlphabet*>(geneticCode_->getTargetAlphabet());
 
-    if (args.find("genetic_code") == args.end())
-      args["genetic_code"] = pWA->getAlphabetType();
-
-    GeneticCode* pgc = SequenceApplicationTools::getGeneticCode(dynamic_cast<const NucleicAlphabet*>(pWA->getNAlphabet(0)), args["genetic_code"]);
-    if (pgc->getSourceAlphabet()->getAlphabetType() != pWA->getAlphabetType())
-      throw Exception("Mismatch between genetic code and codon alphabet");
-
-    const ProteicAlphabet* pPA = dynamic_cast<const ProteicAlphabet*>(pgc->getTargetAlphabet());
-
-    auto_ptr<ProteinFrequenciesSet> pPFS;
+    unsigned short method=1;
+    if (args.find("method") != args.end()){
+      if (args["method"]=="local")
+        method=2;
+      else
+        if (args["method"]=="binary")
+          method=3;
+    }
 
     if (args.find("protein_frequencies") != args.end())
     {
       string sPFS = args["protein_frequencies"];
-      BppOFrequenciesSetFormat nestedReader(alphabetCode_, false);
-      pPFS.reset(dynamic_cast<ProteinFrequenciesSet*>(nestedReader.read(pPA, sPFS, data, false)));
+      BppOFrequenciesSetFormat nestedReader(alphabetCode_, false, warningLevel_);
+      auto_ptr<ProteinFrequenciesSet> pPFS(dynamic_cast<ProteinFrequenciesSet*>(nestedReader.read(pPA, sPFS, data, false)));
       map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
 
       for (map<string, string>::iterator it = unparsedParameterValuesNested.begin(); it != unparsedParameterValuesNested.end(); it++)
       {
         unparsedArguments_["FullPerAA." + it->first] = it->second;
       }
-      pFS.reset(new FullPerAACodonFrequenciesSet(pgc, pPFS.release()));
+      pFS.reset(new FullPerAACodonFrequenciesSet(geneticCode_, pPFS.release(), method));
     }
     else
-      pFS.reset(new FullPerAACodonFrequenciesSet(pgc));
+      pFS.reset(new FullPerAACodonFrequenciesSet(geneticCode_, method));
   }
 
   // codeml frequencies syntax
@@ -311,10 +333,17 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
   {
     if (!(alphabetCode_ & CODON))
       throw Exception("Codon alphabet not supported.");
+    if (!geneticCode_)
+      throw Exception("BppOFrequenciesSetFormat::read(). No genetic code specified! Consider using 'setGeneticCode'.");
+    
     const CodonAlphabet* pWA = dynamic_cast<const CodonAlphabet*>(alphabet);
 
+    if (args.find("genetic_code") != args.end()) {
+      ApplicationTools::displayWarning("'genetic_code' argument is no longer supported inside model description, and has been supersided by a global 'genetic_code' option.");
+      throw Exception("BppOFrequenciesSetFormat::read. Deprecated 'genetic_code' argument.");
+    }
     short opt = -1;
-    string mgmtStopCodon="quadratic";
+    string mgmtStopCodon = "quadratic";
     
     if (freqName == "F0")
     {
@@ -332,7 +361,7 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
         {
           string sAFS = args["frequency"];
 
-          BppOFrequenciesSetFormat nestedReader(alphabetCode_, false);
+          BppOFrequenciesSetFormat nestedReader(alphabetCode_, false, warningLevel_);
           auto_ptr<FrequenciesSet> pFS2(nestedReader.read(pWA->getNAlphabet(0), sAFS, data, false));
           if (pFS2->getName()!="Full")
             throw Exception("BppOFrequenciesSetFormat::read. The frequency option in F1X4 can only be Full");
@@ -350,6 +379,12 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
         unparsedArguments_["123_Full.theta1"] = args["123_theta1"];
       if (args.find("123_theta2") != args.end())
         unparsedArguments_["123_Full.theta2"] = args["123_theta2"];
+      if (args.find("theta") != args.end())
+        unparsedArguments_["123_Full.theta"] = args["123_theta"];
+      if (args.find("theta1") != args.end())
+        unparsedArguments_["123_Full.theta1"] = args["123_theta1"];
+      if (args.find("theta2") != args.end())
+        unparsedArguments_["123_Full.theta2"] = args["123_theta2"];
     }
     else if (freqName == "F3X4")
     {
@@ -374,7 +409,7 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
         
         for (unsigned i = 0; i < v_sAFS.size(); i++)
           {
-            BppOFrequenciesSetFormat nestedReader(alphabetCode_, false);
+            BppOFrequenciesSetFormat nestedReader(alphabetCode_, false, warningLevel_);
             if (v_sAFS[i]!=""){
               pFS.reset(nestedReader.read(pWA->getNAlphabet(i), v_sAFS[i], data, false));
               if (pFS->getName()!="Full")
@@ -402,8 +437,17 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
     {
       opt = CodonFrequenciesSet::F61;
     }
-    if (opt != -1)
-      pFS.reset(CodonFrequenciesSet::getFrequenciesSetForCodons(opt, *dynamic_cast<const CodonAlphabet*>(alphabet), mgmtStopCodon));
+    if (opt != -1){
+      unsigned short method=1;
+      if (args.find("method") != args.end()){
+        if (args["method"]=="local")
+          method=2;
+        else
+          if (args["method"]=="binary")
+            method=3;
+      }      
+      pFS.reset(CodonFrequenciesSet::getFrequenciesSetForCodons(opt, geneticCode_, mgmtStopCodon, method));
+    }
     else
       throw Exception("Unknown frequency option: " + freqName);
   }
@@ -420,7 +464,7 @@ FrequenciesSet* BppOFrequenciesSetFormat::read(const Alphabet* alphabet, const s
   vector<string> pnames = pFS->getParameters().getParameterNames();
 
   string pref = pFS->getNamespace();
-  for (unsigned int i = 0; i < pnames.size(); i++)
+  for (size_t i = 0; i < pnames.size(); i++)
   {
     string name = pFS->getParameterNameWithoutNamespace(pnames[i]);
     if (args.find(name) != args.end())
@@ -459,7 +503,7 @@ void BppOFrequenciesSetFormat::write(const FrequenciesSet* pfreqset,
     return;
   }
   ParameterList pl = pfreqset->getParameters();
-  unsigned int p = out.getPrecision();
+  int p = out.getPrecision();
   out.setPrecision(12);
   bool flag(false);
   string name = pfreqset->getName();
@@ -470,7 +514,7 @@ void BppOFrequenciesSetFormat::write(const FrequenciesSet* pfreqset,
   {
     vector<double> vf = pfreqset->getFrequencies();
     out << "values=(";
-    for (unsigned int i = 0; i < vf.size(); i++)
+    for (size_t i = 0; i < vf.size(); i++)
     {
       if (i != 0)
         out << ", ";
@@ -480,34 +524,46 @@ void BppOFrequenciesSetFormat::write(const FrequenciesSet* pfreqset,
   }
   else
   {
-    if (name != "F1X4" && name != "F3X4" && name != "F61")
+    if (dynamic_cast<const FullProteinFrequenciesSet*>(pfreqset))
     {
-      const WordFromIndependentFrequenciesSet* pWFI = dynamic_cast<const WordFromIndependentFrequenciesSet*>(pfreqset);
-      if (pWFI != NULL)
-      {
-        for (unsigned int i = 0; i < pWFI->getLength(); i++)
-        {
-          if (i != 0)
-            out << ", ";
-          out << "frequency" << i + 1 << "=";
-          write(&pWFI->getFrequenciesSetForLetter(i), out, writtenNames);
-        }
-        flag = true;
+      size_t meth=dynamic_cast<const FullProteinFrequenciesSet*>(pfreqset)->getMethod();
+      if (meth>1){
+        if (flag)
+          out << ",";
+        out << "method=" << ((meth==2)?"local":"binary");
+        flag=true;
       }
-      const WordFromUniqueFrequenciesSet* pWFU = dynamic_cast<const WordFromUniqueFrequenciesSet*>(pfreqset);
-      if (pWFU != NULL)
+    }
+    else
+    {
+      if (name != "F1X4" && name != "F3X4" && name != "F61")
       {
-        for (unsigned int i = 0; i < pWFU->getLength(); i++)
+        const WordFromIndependentFrequenciesSet* pWFI = dynamic_cast<const WordFromIndependentFrequenciesSet*>(pfreqset);
+        if (pWFI != NULL)
         {
-          if (i != 0)
-            out << ", ";
-          out << "frequency=";
-          write(&pWFU->getFrequenciesSetForLetter(i), out, writtenNames);
+          for (size_t i = 0; i < pWFI->getLength(); i++)
+          {
+            if (i != 0)
+              out << ", ";
+            out << "frequency" << i + 1 << "=";
+            write(&pWFI->getFrequenciesSetForLetter(i), out, writtenNames);
+          }
+          flag = true;
         }
-        flag = true;
-      }
-      const FullPerAACodonFrequenciesSet* pFPA=dynamic_cast<const FullPerAACodonFrequenciesSet*>(pfreqset);
-      if (pFPA != NULL)
+        const WordFromUniqueFrequenciesSet* pWFU = dynamic_cast<const WordFromUniqueFrequenciesSet*>(pfreqset);
+        if (pWFU != NULL)
+        {
+          for (size_t i = 0; i < pWFU->getLength(); i++)
+          {
+            if (i != 0)
+              out << ", ";
+            out << "frequency=";
+            write(&pWFU->getFrequenciesSetForLetter(i), out, writtenNames);
+          }
+          flag = true;
+        }
+        const FullPerAACodonFrequenciesSet* pFPA=dynamic_cast<const FullPerAACodonFrequenciesSet*>(pfreqset);
+        if (pFPA != NULL)
         {
           const ProteinFrequenciesSet* ppfs=pFPA->getProteinFrequenciesSet();
           out << "protein_frequencies=";
@@ -515,22 +571,43 @@ void BppOFrequenciesSetFormat::write(const FrequenciesSet* pfreqset,
           write(ppfs, out, writtenNames);
       
           flag = true;
-        }
-    }
-
-    for (unsigned int i = 0; i < pl.size(); i++)
-      {
-        if (find(writtenNames.begin(), writtenNames.end(), pl[i].getName()) == writtenNames.end())
-          {
+              
+          unsigned short meth=pFPA->getMethod();
+          if (meth>1){
             if (flag)
               out << ",";
-            else
-              flag = true;
-            string pname = pfreqset->getParameterNameWithoutNamespace(pl[i].getName());
-            (out << pname << "=").enableScientificNotation(false) << pl[i].getValue();
-            writtenNames.push_back(pl[i].getName());
+            out << "method=" << ((meth==2)?"local":"binary");
+            flag=true;
           }
+        }
       }
+    }
+
+    
+    for (size_t i = 0; i < pl.size(); i++)
+    {
+      if (find(writtenNames.begin(), writtenNames.end(), pl[i].getName()) == writtenNames.end())
+      {
+        if (flag)
+          out << ",";
+        else
+          flag = true;
+        string pname = pfreqset->getParameterNameWithoutNamespace(pl[i].getName());
+        (out << pname << "=").enableScientificNotation(false) << pl[i].getValue();
+        writtenNames.push_back(pl[i].getName());
+      }
+    }
+    const FullCodonFrequenciesSet* pF=dynamic_cast<const FullCodonFrequenciesSet*>(pfreqset);
+    if (pF != NULL)
+    {
+      unsigned short meth=pF->getMethod();
+      if (meth>1){
+        if (flag)
+          out << ",";
+        out << "method=" << ((meth==2)?"local":"binary");
+        flag=true;
+      }
+    }
   }
   
   out << ")";
@@ -549,12 +626,12 @@ void BppOFrequenciesSetFormat::initialize_(FrequenciesSet& freqSet, const SiteCo
         throw Exception("Missing data for observed frequencies");
       unsigned int psc = 0;
       if (unparsedArguments_.find("observedPseudoCount") != unparsedArguments_.end())
-        psc = TextTools::toInt(unparsedArguments_["observedPseudoCount"]);
+        psc = TextTools::to<unsigned int>(unparsedArguments_["observedPseudoCount"]);
 
       map<int, double> freqs;
       SequenceContainerTools::getFrequencies(*data, freqs, psc);
 
-      freqSet.setFrequenciesFromMap(freqs);
+      freqSet.setFrequenciesFromAlphabetStatesFrequencies(freqs);
     }
     else if (init.substr(0, 6) == "values")
     {
@@ -579,7 +656,7 @@ void BppOFrequenciesSetFormat::initialize_(FrequenciesSet& freqSet, const SiteCo
     // Explicit initialization of each parameter
     ParameterList pl = freqSet.getParameters();
 
-    for (unsigned int i = 0; i < pl.size(); i++)
+    for (size_t i = 0; i < pl.size(); ++i)
     {
       AutoParameter ap(pl[i]);
       if (verbose_)
@@ -587,10 +664,10 @@ void BppOFrequenciesSetFormat::initialize_(FrequenciesSet& freqSet, const SiteCo
       pl.setParameter(i, ap);
     }
 
-    for (unsigned int i = 0; i < pl.size(); i++)
+    for (size_t i = 0; i < pl.size(); ++i)
     {
       const string pName = pl[i].getName();
-      double value = ApplicationTools::getDoubleParameter(pName, unparsedArguments_, pl[i].getValue());
+      double value = ApplicationTools::getDoubleParameter(pName, unparsedArguments_, pl[i].getValue(), "", true, warningLevel_);
 
       pl[i].setValue(value);
       if (verbose_)
