@@ -92,6 +92,7 @@
 #include "../Model/Protein/LGL08_CAT.h"
 #include "../Model/Protein/WAG01.h"
 #include "../Model/Protein/LLG08_EHO.h"
+#include "../Model/Protein/LG10_EX_EHO.h"
 #include "../Model/BinarySubstitutionModel.h"
 
 #include "../App/PhylogeneticsApplicationTools.h"
@@ -127,10 +128,11 @@ unsigned char BppOSubstitutionModelFormat::BINARY = 32;
 unsigned char BppOSubstitutionModelFormat::ALL = 1 | 2 | 4 | 8 | 16 | 32;
 
 
-SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
-                                                     const std::string& modelDescription,
-                                                     const SiteContainer* data,
-                                                     bool parseArguments)
+SubstitutionModel* BppOSubstitutionModelFormat::read(
+    const Alphabet* alphabet,
+    const std::string& modelDescription,
+    const SiteContainer* data,
+    bool parseArguments)
 {
   unparsedArguments_.clear();
   auto_ptr<SubstitutionModel> model;
@@ -163,20 +165,25 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
   {
     if (!(alphabetCode_ & CODON))
       throw Exception("BppOSubstitutionModelFormat::read. Codon alphabet not supported.");
+    if (!geneticCode_)
+      throw Exception("BppOSubstitutionModelFormat::read(). No genetic code specified! Consider using 'setGeneticCode'.");
+    
     if (!AlphabetTools::isCodonAlphabet(alphabet))
       throw Exception("Alphabet should be Codon Alphabet.");
 
     const CodonAlphabet* pCA = dynamic_cast<const CodonAlphabet*>(alphabet);
 
-    if (args.find("genetic_code") == args.end())
-      args["genetic_code"] = pCA->getAlphabetType();
+    if (args.find("genetic_code") != args.end()) {
+      ApplicationTools::displayWarning("'genetic_code' argument is no longer supported inside model description, and has been supersided by a global 'genetic_code' option.");
+      throw Exception("BppOSubstitutionModelFormat::read. Deprecated 'genetic_code' argument.");
+    }
 
-    auto_ptr<GeneticCode> pgc(SequenceApplicationTools::getGeneticCode(dynamic_cast<const NucleicAlphabet*>(pCA->getNAlphabet(0)), args["genetic_code"]));
-    if (pgc->getSourceAlphabet()->getAlphabetType() != pCA->getAlphabetType())
+    if (geneticCode_->getSourceAlphabet()->getAlphabetType() != pCA->getAlphabetType())
       throw Exception("Mismatch between genetic code and codon alphabet");
 
-    string freqOpt = ApplicationTools::getStringParameter("frequencies", args, "F0", "", true, verbose_);
-    BppOFrequenciesSetFormat freqReader(BppOFrequenciesSetFormat::ALL, verbose_);
+    string freqOpt = ApplicationTools::getStringParameter("frequencies", args, "F0", "", true, warningLevel_);
+    BppOFrequenciesSetFormat freqReader(BppOFrequenciesSetFormat::ALL, verbose_, warningLevel_);
+    freqReader.setGeneticCode(geneticCode_); //This uses the same instance as the one that will be used by the model.
     auto_ptr<FrequenciesSet> codonFreqs(freqReader.read(pCA, freqOpt, data, false));
     map<string, string> unparsedParameterValuesNested(freqReader.getUnparsedArguments());
 
@@ -186,20 +193,20 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
     }
 
     if (modelName == "MG94")
-      model.reset(new MG94(pgc.release(), codonFreqs.release()));
+      model.reset(new MG94(geneticCode_, codonFreqs.release()));
     else if (modelName == "GY94")
-      model.reset(new GY94(pgc.release(), codonFreqs.release()));
+      model.reset(new GY94(geneticCode_, codonFreqs.release()));
     else if ((modelName == "YN98") || (modelName == "YNGKP_M0"))
-      model.reset(new YN98(pgc.release(), codonFreqs.release()));
+      model.reset(new YN98(geneticCode_, codonFreqs.release()));
     else if (modelName == "YNGKP_M1")
-      model.reset(new YNGKP_M1(pgc.release(), codonFreqs.release()));
+      model.reset(new YNGKP_M1(geneticCode_, codonFreqs.release()));
     else if (modelName == "YNGKP_M2")
-      model.reset(new YNGKP_M2(pgc.release(), codonFreqs.release()));
+      model.reset(new YNGKP_M2(geneticCode_, codonFreqs.release()));
     else if (modelName == "YNGKP_M3")
       if (args.find("n") == args.end())
-        model.reset(new YNGKP_M3(pgc.release(), codonFreqs.release()));
+        model.reset(new YNGKP_M3(geneticCode_, codonFreqs.release()));
       else
-        model.reset(new YNGKP_M3(pgc.release(), codonFreqs.release(), TextTools::to<unsigned int>(args["n"])));
+        model.reset(new YNGKP_M3(geneticCode_, codonFreqs.release(), TextTools::to<unsigned int>(args["n"])));
     else if ((modelName == "YNGKP_M7") || modelName == "YNGKP_M8")
     {
       if (args.find("n") == args.end())
@@ -209,9 +216,9 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
         ApplicationTools::displayResult("Number of classes in model", nbClasses);
 
       if (modelName == "YNGKP_M7")
-        model.reset(new YNGKP_M7(pgc.release(), codonFreqs.release(), nbClasses));
+        model.reset(new YNGKP_M7(geneticCode_, codonFreqs.release(), nbClasses));
       else if (modelName == "YNGKP_M8")
-        model.reset(new YNGKP_M8(pgc.release(), codonFreqs.release(), nbClasses));
+        model.reset(new YNGKP_M8(geneticCode_, codonFreqs.release(), nbClasses));
     }
     else
       throw Exception("Unknown Codon model: " + modelName);
@@ -232,7 +239,7 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
       throw Exception("BppOSubstitutionModelFormat::read. Missing argument 'model' for model 'gBGC'.");
     if (verbose_)
       ApplicationTools::displayResult("Biased gene conversion", modelName);
-    BppOSubstitutionModelFormat nestedReader(NUCLEOTIDE, true, true, false, verbose_);
+    BppOSubstitutionModelFormat nestedReader(NUCLEOTIDE, true, true, false, verbose_, warningLevel_);
     auto_ptr<NucleotideSubstitutionModel> nestedModel(dynamic_cast<NucleotideSubstitutionModel*>(nestedReader.read(alphabet, nestedModelDescription, data, false)));
     map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
 
@@ -263,7 +270,7 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
       throw Exception("BppOSubstitutionModelFormat::read. Missing argument 'model' for model 'YpR_sym'.");
     if (verbose_)
       ApplicationTools::displayResult("Symetric YpR model", modelName);
-    BppOSubstitutionModelFormat nestedReader(NUCLEOTIDE, false, false, false, verbose_);
+    BppOSubstitutionModelFormat nestedReader(NUCLEOTIDE, false, false, false, verbose_, warningLevel_);
     auto_ptr<NucleotideSubstitutionModel> nestedModel(dynamic_cast<NucleotideSubstitutionModel*>(nestedReader.read(&prny->getLetterAlphabet(), nestedModelDescription, data, false)));
     map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
     for (map<string, string>::iterator it = unparsedParameterValuesNested.begin(); it != unparsedParameterValuesNested.end(); it++)
@@ -286,7 +293,7 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
       throw Exception("BppOSubstitutionModelFormat::read. Missing argument 'model' for model 'YpR_gen'.");
     if (verbose_)
       ApplicationTools::displayResult("General YpR model", modelName);
-    BppOSubstitutionModelFormat nestedReader(NUCLEOTIDE, false, false, false, verbose_);
+    BppOSubstitutionModelFormat nestedReader(NUCLEOTIDE, false, false, false, verbose_, warningLevel_);
     auto_ptr<NucleotideSubstitutionModel> nestedModel(dynamic_cast<NucleotideSubstitutionModel*>(nestedReader.read(&prny->getLetterAlphabet(), nestedModelDescription, data, false)));
     map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
 
@@ -314,7 +321,7 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
       throw Exception("BppOSubstitutionModelFormat::read. Missing argument 'model' for model 'RE08'.");
     if (verbose_)
       ApplicationTools::displayResult("Gap model", modelName);
-    BppOSubstitutionModelFormat nestedReader(ALL, allowCovarions_, false, false, verbose_);
+    BppOSubstitutionModelFormat nestedReader(ALL, allowCovarions_, false, false, verbose_, warningLevel_);
     auto_ptr<ReversibleSubstitutionModel> nestedModel(dynamic_cast<ReversibleSubstitutionModel*>(nestedReader.read(alphabet, nestedModelDescription, data, false)));
     map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
 
@@ -343,7 +350,7 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
       throw Exception("BppOSubstitutionModelFormat::read. Missing argument 'model' for model 'TS98'.");
     if (verbose_)
       ApplicationTools::displayResult("Covarion model", modelName);
-    BppOSubstitutionModelFormat nestedReader(ALL, false, allowMixed_, allowGaps_, false);
+    BppOSubstitutionModelFormat nestedReader(ALL, false, allowMixed_, allowGaps_, false, warningLevel_);
     auto_ptr<ReversibleSubstitutionModel> nestedModel(dynamic_cast<ReversibleSubstitutionModel*>(nestedReader.read(alphabet, nestedModelDescription, data, false)));
     map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
 
@@ -375,7 +382,7 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
       throw Exception("BppOSubstitutionModelFormat::read. Missing argument 'rdist' for model 'G01'.");
     if (verbose_)
       ApplicationTools::displayResult("Covarion model", modelName);
-    BppOSubstitutionModelFormat nestedReader(ALL, false, allowMixed_, allowGaps_, verbose_);
+    BppOSubstitutionModelFormat nestedReader(ALL, false, allowMixed_, allowGaps_, verbose_, warningLevel_);
     auto_ptr<ReversibleSubstitutionModel> nestedModel(dynamic_cast<ReversibleSubstitutionModel*>(nestedReader.read(alphabet, nestedModelDescription, data, false)));
     map<string, string> unparsedParameterValuesNestedModel(nestedReader.getUnparsedArguments());
     BppORateDistributionFormat rateReader(false);
@@ -515,9 +522,9 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
         throw Exception("BppOSubstitutionModelFormat::read. Protein alphabet not supported.");
       const ProteicAlphabet* alpha = dynamic_cast<const ProteicAlphabet*>(alphabet);
 
-      if (modelName.find("+F")!=string::npos){
-        string freqOpt = ApplicationTools::getStringParameter("frequencies", args, "Full", "", true, verbose_);
-        BppOFrequenciesSetFormat freqReader(BppOFrequenciesSetFormat::ALL, false);
+      if (modelName.find("+F") != string::npos) {
+        string freqOpt = ApplicationTools::getStringParameter("frequencies", args, "Full", "", true, warningLevel_);
+        BppOFrequenciesSetFormat freqReader(BppOFrequenciesSetFormat::ALL, false, warningLevel_);
         auto_ptr<FrequenciesSet> protFreq(freqReader.read(alpha, freqOpt, data, true));
         map<string, string> unparsedParameterValuesNested(freqReader.getUnparsedArguments());
 
@@ -564,9 +571,11 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
         model.reset(new LLG08_UL2(alpha));
       else if (modelName == "LLG08_UL3")
         model.reset(new LLG08_UL3(alpha));
+      else if (modelName == "LG10_EX_EHO")
+        model.reset(new LG10_EX_EHO(alpha));	
       else if (modelName == "LGL08_CAT")
       {
-        unsigned int nbCat = TextTools::toInt(args["nbCat"]);
+        unsigned int nbCat = TextTools::to<unsigned int>(args["nbCat"]);
         model.reset(new LGL08_CAT(alpha, nbCat));
       }
       else if (modelName == "Empirical")
@@ -576,19 +585,22 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
           throw Exception("'name' argument missing for user-defined substitution model.");
         model.reset(new UserProteinSubstitutionModel(alpha, args["file"], prefix));
       }
-      else if (modelName == "COaLA")
+      else if (modelName == "Coala")
       {
+        string exchangeability = args["exch"];
+        if (TextTools::isEmpty(exchangeability))
+          throw Exception("BppOSubstitutionModelFormat::read. missing argument 'exch' for model 'Coala'.");
+        string prefix = args["name"];
+        if (exchangeability == "Empirical" && TextTools::isEmpty(prefix))
+          throw Exception("'name' argument missing to specify the exchangeabilities of the user-defined empirical model.");  
+        BppOSubstitutionModelFormat nestedReader(PROTEIN, false, allowMixed_, allowGaps_, verbose_, warningLevel_);
+        auto_ptr<ProteinSubstitutionModel> nestedModel(dynamic_cast<ProteinSubstitutionModel*>(nestedReader.read(alphabet, exchangeability, data, false)));
         string nbrOfParametersPerBranch = args["nbrAxes"];
         if (TextTools::isEmpty(nbrOfParametersPerBranch))
           throw Exception("'nbrAxes' argument missing to define the number of axis of the Correspondence Analysis.");
-        string exchangeability = args["exch"];
-        if (TextTools::isEmpty(exchangeability))
-          throw Exception("'exch' argument missing to define exchangeability.");
-        string file = args["file"];
-        if (exchangeability == "Empirical" && TextTools::isEmpty(file))
-          throw Exception("'file' argument missing to specify the exchangeabilities of the user-defined empirical model.");
-        model.reset(new Coala(alpha, TextTools::toInt(nbrOfParametersPerBranch), exchangeability, file));
-        dynamic_cast<Coala*>(model.get())->setFreqFromData(*data);
+        //Now we create the Coala model:
+        model.reset(new Coala(alpha, *nestedModel, TextTools::to<unsigned int>(nbrOfParametersPerBranch)));
+        model->setFreqFromData(*data);
       }
 
       else if (AlphabetTools::isBinaryAlphabet(alphabet))
@@ -630,7 +642,7 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
       continue;
     pval = args[pname];
 
-    if ((pval.length() >= 5 && pval.substr(0, 5) == "model") ||
+    if (((pval.rfind("_") != string::npos) && (TextTools::isDecimalInteger(pval.substr(pval.rfind("_")+1,string::npos)))) ||
         (pval.find("(") != string::npos))
       continue;
     bool found = false;
@@ -651,8 +663,8 @@ SubstitutionModel* BppOSubstitutionModelFormat::read(const Alphabet* alphabet,
         found = true;
       }
     }
-    if (!TextTools::isDecimalNumber(pval) && !found)
-      throw Exception("Incorrect parameter syntax: parameter " + pval + " was not found and can't be used as a value for parameter " + pname + ".");
+    // if (!TextTools::isDecimalNumber(pval) && !found)
+    //   throw Exception("Incorrect parameter syntax: parameter " + pval + " was not found and can't be used as a value for parameter " + pname + ".");
   }
 
   // 2 following tests be removed in a later version
@@ -721,7 +733,7 @@ SubstitutionModel* BppOSubstitutionModelFormat::readWord_(const Alphabet* alphab
 
   if (v_nestedModelDescription.size() != nbmodels)
   {
-    BppOSubstitutionModelFormat nestedReader(alphabetCode_, false, true, false, false);
+    BppOSubstitutionModelFormat nestedReader(alphabetCode_, false, true, false, false, warningLevel_);
     model.reset(nestedReader.read(pWA->getNAlphabet(0), v_nestedModelDescription[0], data, false));
     map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
     string pref = "";
@@ -741,7 +753,7 @@ SubstitutionModel* BppOSubstitutionModelFormat::readWord_(const Alphabet* alphab
   {
     for (unsigned i = 0; i < v_nestedModelDescription.size(); i++)
     {
-      BppOSubstitutionModelFormat nestedReader(alphabetCode_, false, true, false, false);
+      BppOSubstitutionModelFormat nestedReader(alphabetCode_, false, true, false, false, warningLevel_);
       model.reset(nestedReader.read(pWA->getNAlphabet(i), v_nestedModelDescription[i], data, false));
       map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
       for (map<string, string>::iterator it = unparsedParameterValuesNested.begin(); it != unparsedParameterValuesNested.end(); it++)
@@ -772,10 +784,9 @@ SubstitutionModel* BppOSubstitutionModelFormat::readWord_(const Alphabet* alphab
   {
     const CodonAlphabet* pCA = dynamic_cast<const CodonAlphabet*>(pWA);
     if (pCA == 0)
-      throw Exception("Non codon Alphabet fo model" + modelName + " model.");
+      throw Exception("Non codon Alphabet for model" + modelName + ".");
 
     auto_ptr< AlphabetIndex2 > pai2;
-    auto_ptr<GeneticCode> pgc;
     auto_ptr<FrequenciesSet> pFS;
 
     if ((dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]) == 0) ||
@@ -783,26 +794,26 @@ SubstitutionModel* BppOSubstitutionModelFormat::readWord_(const Alphabet* alphab
          (dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[1]) == 0 || dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[2]) == 0)))
       throw Exception("Non simple NucleotideSubstitutionModel imbedded in " + modelName + " model.");
 
-
-    if (modelName.find("Dist") != string::npos)
-    {
-      if (args.find("genetic_code") == args.end())
-        args["genetic_code"] = pCA->getAlphabetType();
-
-      pgc.reset(SequenceApplicationTools::getGeneticCode(dynamic_cast<const NucleicAlphabet*>(pCA->getNAlphabet(0)), args["genetic_code"]));
-      if (pgc->getSourceAlphabet()->getAlphabetType() != pCA->getAlphabetType())
-        throw Exception("Mismatch between genetic code and codon alphabet");
-
-      pai2.reset((args.find("aadistance") == args.end()) ? 0 : SequenceApplicationTools::getAlphabetIndex2(&AlphabetTools::PROTEIN_ALPHABET, args["aadistance"]));
+    if (args.find("genetic_code") != args.end()) {
+      ApplicationTools::displayWarning("'genetic_code' argument is no longer supported inside model description, and has been supersided by a global 'genetic_code' option.");
+      throw Exception("BppOSubstitutionModelFormat::read. Deprecated 'genetic_code' argument.");
     }
 
+    if (!geneticCode_)
+      throw Exception("BppOSubstitutionModelFormat::readWord_(). No genetic code specified! Consider using 'setGeneticCode'.");
+
+    
+    if (modelName.find("Dist") != string::npos)
+      pai2.reset((args.find("aadistance") == args.end()) ? 0 : SequenceApplicationTools::getAlphabetIndex2(&AlphabetTools::PROTEIN_ALPHABET, args["aadistance"]));
+    
 
     if (modelName.find("Freq") != string::npos)
     {
       if (args.find("frequencies") == args.end())
         throw Exception("Missing equilibrium frequencies.");
 
-      BppOFrequenciesSetFormat bIOFreq(alphabetCode_, verbose_);
+      BppOFrequenciesSetFormat bIOFreq(alphabetCode_, verbose_, warningLevel_);
+      bIOFreq.setGeneticCode(geneticCode_); //This uses the same instance as the one that will be used by the model
       pFS.reset(bIOFreq.read(pCA, args["frequencies"], data, false));
       map<string, string> unparsedParameterValuesNested(bIOFreq.getUnparsedArguments());
 
@@ -829,10 +840,10 @@ SubstitutionModel* BppOSubstitutionModelFormat::readWord_(const Alphabet* alphab
     else if (modelName == "CodonRate")
       model.reset((v_nestedModelDescription.size() != 3)
                   ? new CodonRateSubstitutionModel(
-                    pCA,
+                    geneticCode_,
                     dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]))
                   : new CodonRateSubstitutionModel(
-                    pCA,
+                    geneticCode_,
                     dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]),
                     dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[1]),
                     dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[2])));
@@ -841,10 +852,10 @@ SubstitutionModel* BppOSubstitutionModelFormat::readWord_(const Alphabet* alphab
     else if (modelName == "CodonDist")
     {
       if (v_nestedModelDescription.size() != 3)
-        model.reset(new CodonDistanceSubstitutionModel(pgc.release(), dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]), pai2.release()));
+        model.reset(new CodonDistanceSubstitutionModel(geneticCode_, dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]), pai2.release()));
       else
         model.reset(new CodonDistanceSubstitutionModel(
-                      pgc.release(),
+                      geneticCode_,
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]),
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[1]),
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[2]), pai2.release()));
@@ -853,25 +864,31 @@ SubstitutionModel* BppOSubstitutionModelFormat::readWord_(const Alphabet* alphab
     else if (modelName == "CodonRateFreq")
     {
       if (v_nestedModelDescription.size() != 3)
-        model.reset(new CodonRateFrequenciesSubstitutionModel(pCA, dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]), pFS.release()));
+        model.reset(
+            new CodonRateFrequenciesSubstitutionModel(
+              geneticCode_,
+              dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]),
+              pFS.release()));
       else
-        model.reset(new CodonRateFrequenciesSubstitutionModel(pCA,
-                                                              dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]),
-                                                              dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[1]),
-                                                              dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[2]),
-                                                              pFS.release()));
+        model.reset(
+            new CodonRateFrequenciesSubstitutionModel(
+              geneticCode_,
+              dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]),
+              dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[1]),
+              dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[2]),
+              pFS.release()));
     }
 
     else if (modelName == "CodonDistFreq")
     {
       if (v_nestedModelDescription.size() != 3)
-        model.reset(new CodonDistanceFrequenciesSubstitutionModel(pgc.release(),
+        model.reset(new CodonDistanceFrequenciesSubstitutionModel(geneticCode_,
                                                                   dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]),
                                                                   pFS.release(),
                                                                   pai2.release()));
       else
         model.reset(new CodonDistanceFrequenciesSubstitutionModel(
-                      pgc.release(),
+                      geneticCode_,
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]),
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[1]),
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[2]),
@@ -882,13 +899,13 @@ SubstitutionModel* BppOSubstitutionModelFormat::readWord_(const Alphabet* alphab
     else if (modelName == "CodonDistPhasFreq")
     {
       if (v_nestedModelDescription.size() != 3)
-        model.reset(new CodonDistancePhaseFrequenciesSubstitutionModel(pgc.release(),
+        model.reset(new CodonDistancePhaseFrequenciesSubstitutionModel(geneticCode_,
                                                                        dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]),
                                                                        pFS.release(),
                                                                        pai2.release()));
       else
         model.reset(new CodonDistancePhaseFrequenciesSubstitutionModel(
-                      pgc.release(),
+                      geneticCode_,
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]),
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[1]),
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[2]),
@@ -900,7 +917,8 @@ SubstitutionModel* BppOSubstitutionModelFormat::readWord_(const Alphabet* alphab
       if (args.find("fitness") == args.end())
         throw Exception("Missing fitness in model " + modelName + ".");
 
-      BppOFrequenciesSetFormat bIOFreq(alphabetCode_, verbose_);
+      BppOFrequenciesSetFormat bIOFreq(alphabetCode_, verbose_, warningLevel_);
+      bIOFreq.setGeneticCode(geneticCode_); 
       auto_ptr<FrequenciesSet> pFit(bIOFreq.read(pCA, args["fitness"], data, false));
       map<string, string> unparsedParameterValuesNested(bIOFreq.getUnparsedArguments());
 
@@ -911,7 +929,7 @@ SubstitutionModel* BppOSubstitutionModelFormat::readWord_(const Alphabet* alphab
 
       if (v_nestedModelDescription.size() != 3)
       {
-        model.reset(new CodonDistanceFitnessPhaseFrequenciesSubstitutionModel(pgc.release(),
+        model.reset(new CodonDistanceFitnessPhaseFrequenciesSubstitutionModel(geneticCode_,
                                                                               dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]),
                                                                               pFit.release(),
                                                                               pFS.release(),
@@ -919,7 +937,7 @@ SubstitutionModel* BppOSubstitutionModelFormat::readWord_(const Alphabet* alphab
       }
       else
         model.reset(new CodonDistanceFitnessPhaseFrequenciesSubstitutionModel(
-                      pgc.release(),
+                      geneticCode_,
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[0]),
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[1]),
                       dynamic_cast<NucleotideSubstitutionModel*>(v_pSM[2]),
@@ -946,7 +964,7 @@ MixedSubstitutionModel* BppOSubstitutionModelFormat::readMixed_(const Alphabet*
     if (args.find("model") == args.end())
       throw Exception("The argument 'model' is missing from MixedSubstitutionModel description");
     string nestedModelDescription = args["model"];
-    BppOSubstitutionModelFormat nestedReader(alphabetCode_, allowCovarions_, true, allowGaps_, false);
+    BppOSubstitutionModelFormat nestedReader(alphabetCode_, allowCovarions_, true, allowGaps_, false, warningLevel_);
     pSM.reset(nestedReader.read(alphabet, nestedModelDescription, data, false));
     map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
 
@@ -1028,7 +1046,7 @@ MixedSubstitutionModel* BppOSubstitutionModelFormat::readMixed_(const Alphabet*
 
     for (unsigned i = 0; i < v_nestedModelDescription.size(); i++)
     {
-      BppOSubstitutionModelFormat nestedReader(alphabetCode_, false, true, false, false);
+      BppOSubstitutionModelFormat nestedReader(alphabetCode_, false, true, false, false, warningLevel_);
       pSM.reset(nestedReader.read(alphabet, v_nestedModelDescription[i], data, false));
       map<string, string> unparsedParameterValuesNested(nestedReader.getUnparsedArguments());
       for (map<string, string>::iterator it = unparsedParameterValuesNested.begin(); it != unparsedParameterValuesNested.end(); it++)
@@ -1116,6 +1134,16 @@ void BppOSubstitutionModelFormat::write(const SubstitutionModel& model,
     comma = true;
   }
 
+  // Is it a gBGC model?
+  const gBGC* gbgcModel = dynamic_cast<const gBGC*>(&model);
+  if (gbgcModel)
+  {
+    out << "model=";
+    const SubstitutionModel* nestedModel = gbgcModel->getNestedModel();
+    write(*nestedModel, out, globalAliases, writtenNames);
+    comma = true;
+  }
+
   // Is it a word model?
 
   const AbstractWordSubstitutionModel* wM = dynamic_cast<const AbstractWordSubstitutionModel*>(&model);
@@ -1166,7 +1194,7 @@ void BppOSubstitutionModelFormat::write(const SubstitutionModel& model,
       out << ",";
     out << "frequencies=";
 
-    BppOFrequenciesSetFormat bIOFreq(alphabetCode_, false);
+    BppOFrequenciesSetFormat bIOFreq(alphabetCode_, false, warningLevel_);
     bIOFreq.write(pfs, out, writtenNames);
     comma = true;
   }
@@ -1180,7 +1208,7 @@ void BppOSubstitutionModelFormat::write(const SubstitutionModel& model,
       out << ",";
     out << "fitness=";
 
-    BppOFrequenciesSetFormat bIOFreq(alphabetCode_, false);
+    BppOFrequenciesSetFormat bIOFreq(alphabetCode_, false, warningLevel_);
     bIOFreq.write(pCF->getFitness(), out, writtenNames);
     comma = true;
   }
@@ -1188,6 +1216,7 @@ void BppOSubstitutionModelFormat::write(const SubstitutionModel& model,
   // and the other parameters
 
   BppOParametrizableFormat bIO;
+  
   bIO.write(&model, out, globalAliases, model.getIndependentParameters().getParameterNames(), writtenNames, true, comma);
   out << ")";
 }
@@ -1270,7 +1299,7 @@ void BppOSubstitutionModelFormat::initialize_(
   SubstitutionModel& model,
   const SiteContainer* data) throw (Exception)
 {
-  string initFreqs = ApplicationTools::getStringParameter(model.getNamespace() + "initFreqs", unparsedArguments_, "", "", true, false);
+  string initFreqs = ApplicationTools::getStringParameter(model.getNamespace() + "initFreqs", unparsedArguments_, "", "", true, warningLevel_);
   if (verbose_)
     ApplicationTools::displayResult("External frequencies initialization for model", (initFreqs == "") ? "None" : initFreqs);
 
@@ -1280,7 +1309,7 @@ void BppOSubstitutionModelFormat::initialize_(
     {
       if (!data)
         throw Exception("BppOSubstitutionModelFormat::initialize_(). Missing data for observed frequencies");
-      unsigned int psi = ApplicationTools::getParameter<unsigned int>(model.getNamespace() + "initFreqs.observedPseudoCount", unparsedArguments_, 0);
+      unsigned int psi = ApplicationTools::getParameter<unsigned int>(model.getNamespace() + "initFreqs.observedPseudoCount", unparsedArguments_, 0, "", true, warningLevel_);
       model.setFreqFromData(*data, psi);
     }
     else if (initFreqs.substr(0, 6) == "values")
@@ -1290,7 +1319,7 @@ void BppOSubstitutionModelFormat::initialize_(
 
       string rf = initFreqs.substr(6);
       StringTokenizer strtok(rf.substr(1, rf.length() - 2), ",");
-      unsigned int i = 0;
+      int i = 0;
       while (strtok.hasMoreToken())
         frequencies[i++] = TextTools::toDouble(strtok.nextToken());
       model.setFreq(frequencies);
@@ -1314,16 +1343,25 @@ void BppOSubstitutionModelFormat::initialize_(
     bool test1 = (initFreqs == "");
     bool test2 = (model.getParameterNameWithoutNamespace(pName).size() < posp + 6) || (model.getParameterNameWithoutNamespace(pName).substr(posp + 1, 5) != "theta");
     bool test3 = (unparsedArguments_.find(pName) != unparsedArguments_.end());
-    if (test1 || test2 || test3)
-    {
-      if (!test1 && !test2 && test3)
-        ApplicationTools::displayWarning("Warning, initFreqs argument is set and a value is set for parameter " + pName);
-      double value = ApplicationTools::getDoubleParameter(pName, unparsedArguments_, pl[i].getValue());
-      pl[i].setValue(value);
+    try {
+      if (test1 || test2 || test3)
+      {
+        if (!test1 && !test2 && test3)
+          ApplicationTools::displayWarning("Warning, initFreqs argument is set and a value is set for parameter " + pName);
+
+        double value = ApplicationTools::getDoubleParameter(pName, unparsedArguments_, pl[i].getValue(), "", true, warningLevel_);
+        pl[i].setValue(value);
+        
+        if (unparsedArguments_.find(pName) != unparsedArguments_.end())
+          unparsedArguments_.erase(unparsedArguments_.find(pName));
+      }
+      if (verbose_)
+        ApplicationTools::displayResult("Parameter found", pName + "=" + TextTools::toString(pl[i].getValue()));
     }
-    if (verbose_)
-      ApplicationTools::displayResult("Parameter found", pName + "=" + TextTools::toString(pl[i].getValue()));
+  
+    catch (Exception& e) {}
   }
+  
 
   model.matchParametersValues(pl);
 }
