@@ -269,7 +269,7 @@ double DRHomogeneousMixedTreeLikelihood::getLogLikelihood() const
 }
 
 
-double DRHomogeneousMixedTreeLikelihood::getLikelihoodForASite(unsigned int site) const
+double DRHomogeneousMixedTreeLikelihood::getLikelihoodForASite(size_t site) const
 {
   double res = 0;
   for (unsigned int i = 0; i < treeLikelihoodsContainer_.size(); i++)
@@ -280,14 +280,14 @@ double DRHomogeneousMixedTreeLikelihood::getLikelihoodForASite(unsigned int site
   return res;
 }
 
-double DRHomogeneousMixedTreeLikelihood::getLogLikelihoodForASite(unsigned int site) const
+double DRHomogeneousMixedTreeLikelihood::getLogLikelihoodForASite(size_t site) const
 {
   double x = getLikelihoodForASite(site);
   if (x < 0) x = 0;
   return log(x);
 }
 
-double DRHomogeneousMixedTreeLikelihood::getLikelihoodForASiteForARateClass(unsigned int site, unsigned int rateClass) const
+double DRHomogeneousMixedTreeLikelihood::getLikelihoodForASiteForARateClass(size_t site, size_t rateClass) const
 {
   double res = 0;
   for (unsigned int i = 0; i < treeLikelihoodsContainer_.size(); i++)
@@ -298,14 +298,14 @@ double DRHomogeneousMixedTreeLikelihood::getLikelihoodForASiteForARateClass(unsi
   return res;
 }
 
-double DRHomogeneousMixedTreeLikelihood::getLogLikelihoodForASiteForARateClass(unsigned int site, unsigned int rateClass) const
+double DRHomogeneousMixedTreeLikelihood::getLogLikelihoodForASiteForARateClass(size_t site, size_t rateClass) const
 {
   double x = getLikelihoodForASiteForARateClass(site, rateClass);
   if (x < 0) x = 0;
   return log(x);
 }
 
-double DRHomogeneousMixedTreeLikelihood::getLikelihoodForASiteForARateClassForAState(unsigned int site, unsigned int rateClass, int state) const
+double DRHomogeneousMixedTreeLikelihood::getLikelihoodForASiteForARateClassForAState(size_t site, size_t rateClass, int state) const
 {
   double res = 0;
 
@@ -317,7 +317,7 @@ double DRHomogeneousMixedTreeLikelihood::getLikelihoodForASiteForARateClassForAS
   return res;
 }
 
-double DRHomogeneousMixedTreeLikelihood::getLogLikelihoodForASiteForARateClassForAState(unsigned int site, unsigned int rateClass, int state) const
+double DRHomogeneousMixedTreeLikelihood::getLogLikelihoodForASiteForARateClassForAState(size_t site, size_t rateClass, int state) const
 {
   double x = getLikelihoodForASiteForARateClassForAState(site, rateClass, state);
   if (x < 0) x = 0;
@@ -343,41 +343,41 @@ void DRHomogeneousMixedTreeLikelihood::computeSubtreeLikelihoodPrefix(const Node
 
 void DRHomogeneousMixedTreeLikelihood::computeRootLikelihood()
 {
-  for (unsigned int i = 0; i < treeLikelihoodsContainer_.size(); i++)
+  for (size_t i = 0; i < treeLikelihoodsContainer_.size(); i++)
   {
     treeLikelihoodsContainer_[i]->computeRootLikelihood();
   }
 }
 
-void DRHomogeneousMixedTreeLikelihood::computeLikelihoodAtNode_(const Node* node, VVVdouble& likelihoodArray) const
+void DRHomogeneousMixedTreeLikelihood::computeLikelihoodAtNode_(const Node* node, VVVdouble& likelihoodArray, const Node* sonNode) const
 {
   likelihoodArray.resize(nbDistinctSites_);
-  for (unsigned int i = 0; i < nbDistinctSites_; i++){
+  for (size_t i = 0; i < nbDistinctSites_; i++){
     VVdouble* likelihoodArray_i = &likelihoodArray[i];
     likelihoodArray_i->resize(nbClasses_);
-    for (unsigned int c = 0; c < nbClasses_; c++) {
+    for (size_t c = 0; c < nbClasses_; c++) {
       Vdouble* likelihoodArray_i_c = &(*likelihoodArray_i)[c];
       likelihoodArray_i_c->resize(nbStates_);
-      for (unsigned int x = 0; x < nbStates_; x++)
+      for (size_t x = 0; x < nbStates_; x++)
         (*likelihoodArray_i_c)[x] = 0;
     }
   }
 
   VVVdouble lArray;
-  for (unsigned int nm = 0; nm < treeLikelihoodsContainer_.size(); nm++)
+  for (size_t nm = 0; nm < treeLikelihoodsContainer_.size(); nm++)
   {
-    treeLikelihoodsContainer_[nm]->computeLikelihoodAtNode_(node, lArray);
+    treeLikelihoodsContainer_[nm]->computeLikelihoodAtNode_(node, lArray, sonNode);
     
-    for (unsigned int i = 0; i < nbDistinctSites_; i++)
+    for (size_t i = 0; i < nbDistinctSites_; i++)
       {
         VVdouble* likelihoodArray_i = &likelihoodArray[i];
         VVdouble* lArray_i = &lArray[i];
         
-        for (unsigned int c = 0; c < nbClasses_; c++)
+        for (size_t c = 0; c < nbClasses_; c++)
           {
             Vdouble* likelihoodArray_i_c = &(*likelihoodArray_i)[c];
             Vdouble* lArray_i_c = &(*lArray_i)[c];
-            for (unsigned int x = 0; x < nbStates_; x++)
+            for (size_t x = 0; x < nbStates_; x++)
               (*likelihoodArray_i_c)[x] += (*lArray_i_c)[x] * probas_[nm];
          }
       }
@@ -391,7 +391,7 @@ void DRHomogeneousMixedTreeLikelihood::computeLikelihoodAtNode_(const Node* node
 
 void DRHomogeneousMixedTreeLikelihood::computeTreeDLikelihoods()
 {
-  for (unsigned int i = 0; i < treeLikelihoodsContainer_.size(); i++)
+  for (size_t i = 0; i < treeLikelihoodsContainer_.size(); i++)
   {
     treeLikelihoodsContainer_[i]->computeTreeDLikelihoods();
   }
@@ -419,7 +419,7 @@ throw (Exception)
   unsigned int brI = TextTools::to<unsigned int>(variable.substr(5));
   const Node* branch = nodes_[brI];
   vector< Vdouble*> _vdLikelihoods_branch;
-  for (unsigned int i = 0; i < treeLikelihoodsContainer_.size(); i++)
+  for (size_t i = 0; i < treeLikelihoodsContainer_.size(); i++)
   {
     _vdLikelihoods_branch.push_back(&treeLikelihoodsContainer_[i]->likelihoodData_->getDLikelihoodArray(branch->getId()));
   }
