@@ -53,12 +53,12 @@ vector<size_t> MarginalAncestralStateReconstruction::getAncestralStatesForNode(i
   if (likelihood_->getTree().isLeaf(nodeId))
   {
     VVdouble larray = likelihood_->getLikelihoodData()->getLeafLikelihoods(nodeId);
-    for (size_t i = 0; i < nbDistinctSites_; i++)
+    for (size_t i = 0; i < nbDistinctSites_; ++i)
     {
       Vdouble* probs_i = &probs[i];
       probs_i->resize(nbStates_);
       size_t j = VectorTools::whichMax(larray[i]);
-      ancestors[i] = (int)j;
+      ancestors[i] = j;
       (*probs_i)[j] = 1.;
     }
   }
@@ -89,7 +89,7 @@ vector<size_t> MarginalAncestralStateReconstruction::getAncestralStatesForNode(i
           cumProb += (*probs_i)[j];
           if (r <= cumProb)
           {
-            ancestors[i] = (int)j;
+            ancestors[i] = j;
             break;
           }
         }
@@ -125,7 +125,7 @@ Sequence* MarginalAncestralStateReconstruction::getAncestralSequenceForNode(int
     probs->resize(nbSites_);
     for (size_t i = 0; i < nbSites_; i++)
     {
-      allStates[i] = model->getAlphabetChar(states[(*rootPatternLinks)[i]]);
+      allStates[i] = model->getAlphabetStateAsInt(states[(*rootPatternLinks)[i]]);
       (*probs)[i] = patternedProbs[(*rootPatternLinks)[i]];
     }
   }
@@ -134,7 +134,7 @@ Sequence* MarginalAncestralStateReconstruction::getAncestralSequenceForNode(int
     states = getAncestralStatesForNode(nodeId, patternedProbs, sample);
     for (size_t i = 0; i < nbSites_; i++)
     {
-      allStates[i] = model->getAlphabetChar(states[(*rootPatternLinks)[i]]);
+      allStates[i] = model->getAlphabetStateAsInt(states[(*rootPatternLinks)[i]]);
     }
   }
   return new BasicSequence(name, allStates, alphabet_);
