@@ -77,12 +77,14 @@ DecompositionSubstitutionCount::DecompositionSubstitutionCount(const ReversibleS
 
 void DecompositionSubstitutionCount::fillBMatrices_()
 {
-  int n = static_cast<int>(nbStates_); //Note jdutheil 20/01/13: shoudl be generalized in case sattes are not 0:n !
-  for (int j = 0; j < n; ++j) {
-    for (int k = 0; k < n; ++k) {
+  vector<int> supportedStates = model_->getAlphabetStates();
+  for (size_t j = 0; j < nbStates_; ++j) {
+    for (size_t k = 0; k < nbStates_; ++k) {
       size_t i = register_->getType(j, k);
       if (i > 0 && k != j) {
-        bMatrices_[i - 1](j, k) = model_->Qij(j, k) * (weights_ ? weights_->getIndex(j, k) : 1);
+        //jdutheil on 25/07/14: I think this is incorrect, weights should only come at the end.
+        //bMatrices_[i - 1](j, k) = model_->Qij(j, k) * (weights_ ? weights_->getIndex(fromState, toState) : 1);
+        bMatrices_[i - 1](j, k) = model_->Qij(j, k);
       }
     }
   }
@@ -90,9 +92,9 @@ void DecompositionSubstitutionCount::fillBMatrices_()
 
 void DecompositionSubstitutionCount::computeEigen_()
 {
-	v_      = model_->getColumnRightEigenVectors();
+  v_      = model_->getColumnRightEigenVectors();
   vInv_   = model_->getRowLeftEigenVectors();
-	lambda_ = model_->getEigenValues();
+  lambda_ = model_->getEigenValues();
 }
 
 void DecompositionSubstitutionCount::computeProducts_()
@@ -157,7 +159,8 @@ void DecompositionSubstitutionCount::computeCounts_(double length) const
     MatrixTools::mult(v_, tmp1, tmp2);
     MatrixTools::mult(tmp2, vInv_, counts_[i]);
 	}
-  // Now we must divide by pijt:
+  // Now we must divide by pijt and account for putative weights:
+  vector<int> supportedStates = model_->getAlphabetStates();
   RowMatrix<double> P = model_->getPij_t(length);
   for (size_t i = 0; i < register_->getNumberOfSubstitutionTypes(); i++) {
     for (size_t j = 0; j < nbStates_; j++) {
@@ -165,6 +168,9 @@ void DecompositionSubstitutionCount::computeCounts_(double length) const
         counts_[i](j, k) /= P(j, k);
         if (isnan(counts_[i](j, k)) || counts_[i](j, k) < 0.) {
           counts_[i](j, k) = 0.;
+        //Weights:
+        if (weights_)
+          counts_[i](j, k) *= weights_->getIndex(supportedStates[j], supportedStates[k]);
         }
       }
     }
@@ -203,7 +209,7 @@ Matrix<double>* DecompositionSubstitutionCount::getAllNumbersOfSubstitutions(dou
 
 /******************************************************************************/
 
-double DecompositionSubstitutionCount::getNumberOfSubstitutions(int initialState, int finalState, double length, size_t type) const
+double DecompositionSubstitutionCount::getNumberOfSubstitutions(size_t initialState, size_t finalState, double length, size_t type) const
 {
   if (length < 0)
     throw Exception("DecompositionSubstitutionCount::getNumbersOfSubstitutions. Negative branch length: " + TextTools::toString(length) + ".");
@@ -217,7 +223,7 @@ double DecompositionSubstitutionCount::getNumberOfSubstitutions(int initialState
 
 /******************************************************************************/
 
-std::vector<double> DecompositionSubstitutionCount::getNumberOfSubstitutionsForEachType(int initialState, int finalState, double length) const
+std::vector<double> DecompositionSubstitutionCount::getNumberOfSubstitutionsForEachType(size_t initialState, size_t finalState, double length) const
 {
   if (length < 0)
     throw Exception("DecompositionSubstitutionCount::getNumbersOfSubstitutions. Negative branch length: " + TextTools::toString(length) + ".");
@@ -227,7 +233,7 @@ std::vector<double> DecompositionSubstitutionCount::getNumberOfSubstitutionsForE
     currentLength_ = length;
   }
   std::vector<double> v(getNumberOfSubstitutionTypes());
-  for (unsigned int t = 0; t < getNumberOfSubstitutionTypes(); ++t) {
+  for (size_t t = 0; t < getNumberOfSubstitutionTypes(); ++t) {
     v[t] = counts_[t](initialState, finalState);
   }
   return v;
@@ -284,7 +290,8 @@ void DecompositionSubstitutionCount::weightsHaveChanged() throw (Exception)
   if (weights_->getAlphabet()->getAlphabetType() != register_->getAlphabet()->getAlphabetType())
     throw Exception("DecompositionSubstitutionCount::weightsHaveChanged. Incorrect alphabet type.");
 
-  fillBMatrices_();
+  //jdutheil on 25/07/14: not necessary if weights are only accounted for in the end.
+  //fillBMatrices_();
   
   //Recompute counts:
   if (currentLength_ > 0)
