@@ -74,7 +74,11 @@ UniformizationSubstitutionCount::UniformizationSubstitutionCount(const Substitut
     if (diagQ > miu_)
       miu_ = diagQ;
   }
-}				
+  
+  if (miu_>10000)
+    throw Exception("UniformizationSubstitutionCount::UniformizationSubstitutionCount The maximum diagonal values of generator is above 10000. Abort, chose another mapping method");
+      
+}        
 
 /******************************************************************************/
 
@@ -90,7 +94,7 @@ void UniformizationSubstitutionCount::resetBMatrices_()
 void UniformizationSubstitutionCount::initBMatrices_()
 {
   //Re-initialize all B matrices according to substitution register.
-  for (unsigned int i = 0; i < register_->getNumberOfSubstitutionTypes(); ++i) {
+  for (size_t i = 0; i < register_->getNumberOfSubstitutionTypes(); ++i) {
     bMatrices_[i].resize(nbStates_, nbStates_);
     counts_[i].resize(nbStates_, nbStates_);
   }
@@ -98,12 +102,13 @@ void UniformizationSubstitutionCount::initBMatrices_()
 
 void UniformizationSubstitutionCount::fillBMatrices_()
 {
-  int n = static_cast<int>(nbStates_); //Note jdutheil 20/01/13: shoudl be generalized in case sattes are not 0:n !
-  for (int j = 0; j < n; ++j) {
-    for (int k = 0; k < n; ++k) {
+  for (size_t j = 0; j < nbStates_; ++j) {
+    for (size_t k = 0; k < nbStates_; ++k) {
       size_t i = register_->getType(j, k);
       if (i > 0 && k != j) {
-        bMatrices_[i - 1](j, k) = model_->Qij(j, k) * (weights_ ? weights_->getIndex(j, k) : 1);
+        //jdutheil on 25/07/14: I think this is incorrect, weights should only come at the end.
+        //bMatrices_[i - 1](j, k) = model_->Qij(j, k) * (weights_ ? weights_->getIndex(fromState, toState) : 1);
+        bMatrices_[i - 1](j, k) = model_->Qij(j, k);
       }
     }
   }
@@ -114,35 +119,35 @@ void UniformizationSubstitutionCount::fillBMatrices_()
 
 void UniformizationSubstitutionCount::computeCounts_(double length) const
 {
-	double lam = miu_ * length;
-	RowMatrix<double> I;
+  double lam = miu_ * length;
+  RowMatrix<double> I;
   MatrixTools::getId(nbStates_, I);
   RowMatrix<double> R(model_->getGenerator());
   MatrixTools::scale(R, 1. / miu_);
   MatrixTools::add(R, I);
-	
-	//compute the stopping point
-	//use the tail of Poisson distribution
-	//can be approximated by 4 + 6 * sqrt(lam) + lam
-	size_t nMax = static_cast<size_t>(ceil(4 + 6 * sqrt(lam) + lam));
+  
+  //compute the stopping point
+  //use the tail of Poisson distribution
+  //can be approximated by 4 + 6 * sqrt(lam) + lam
+  size_t nMax = static_cast<size_t>(ceil(4 + 6 * sqrt(lam) + lam));
 
-	//compute the powers of R
-	power_.resize(nMax + 1);
-	power_[0] = I;
-	for (unsigned int i = 1; i < nMax + 1; ++i)
-		MatrixTools::mult(power_[i - 1], R, power_[i]);
+  //compute the powers of R
+  power_.resize(nMax + 1);
+  power_[0] = I;
+  for (size_t i = 1; i < nMax + 1; ++i)
+    MatrixTools::mult(power_[i - 1], R, power_[i]);
 
-  for (unsigned int i = 0; i < register_->getNumberOfSubstitutionTypes(); ++i) {
+  for (size_t i = 0; i < register_->getNumberOfSubstitutionTypes(); ++i) {
     s_[i].resize(nMax + 1);
     MatrixTools::mult(bMatrices_[i], power_[0], s_[i][0]);
     RowMatrix<double> tmp(nbStates_, nbStates_);
-	  for (unsigned int l = 1; l < nMax + 1; ++l) {
+    for (size_t l = 1; l < nMax + 1; ++l) {
       MatrixTools::mult(R, s_[i][l - 1], s_[i][l]);
       MatrixTools::mult(bMatrices_[i], power_[l], tmp);
       MatrixTools::add(s_[i][l], tmp);
     }
     MatrixTools::fill(counts_[i], 0);
-	  for (unsigned int l = 0; l < nMax + 1; ++l) {
+    for (size_t l = 0; l < nMax + 1; ++l) {
       tmp = s_[i][l];
       //double f = (pow(lam, static_cast<double>(l + 1)) * exp(-lam) / static_cast<double>(NumTools::fact(l + 1))) / miu_;
       double logF = static_cast<double>(l + 1) * log(lam) - lam - log(miu_) - NumTools::logFact(static_cast<double>(l + 1));
@@ -151,14 +156,18 @@ void UniformizationSubstitutionCount::computeCounts_(double length) const
     }
   }
 
-  // Now we must divide by pijt:
+  // Now we must divide by pijt and account for putative weights:
+  vector<int> supportedStates = model_->getAlphabetStates();
   RowMatrix<double> P = model_->getPij_t(length);
-  for (unsigned int i = 0; i < register_->getNumberOfSubstitutionTypes(); i++) {
-    for (unsigned int j = 0; j < nbStates_; j++) {
-      for(unsigned int k = 0; k < nbStates_; k++) {
+  for (size_t i = 0; i < register_->getNumberOfSubstitutionTypes(); i++) {
+    for (size_t j = 0; j < nbStates_; j++) {
+      for(size_t k = 0; k < nbStates_; k++) {
         counts_[i](j, k) /= P(j, k);
         if (isnan(counts_[i](j, k)) || counts_[i](j, k) < 0.)
           counts_[i](j, k) = 0;
+        //Weights:
+        if (weights_)
+          counts_[i](j, k) *= weights_->getIndex(supportedStates[j], supportedStates[k]);
       }
     }
   }
@@ -180,7 +189,7 @@ Matrix<double>* UniformizationSubstitutionCount::getAllNumbersOfSubstitutions(do
 
 /******************************************************************************/
 
-double UniformizationSubstitutionCount::getNumberOfSubstitutions(int initialState, int finalState, double length, size_t type) const
+double UniformizationSubstitutionCount::getNumberOfSubstitutions(size_t initialState, size_t finalState, double length, size_t type) const
 {
   if (length < 0)
     throw Exception("UniformizationSubstitutionCount::getNumbersOfSubstitutions. Negative branch length: " + TextTools::toString(length) + ".");
@@ -194,7 +203,7 @@ double UniformizationSubstitutionCount::getNumberOfSubstitutions(int initialStat
 
 /******************************************************************************/
 
-std::vector<double> UniformizationSubstitutionCount::getNumberOfSubstitutionsForEachType(int initialState, int finalState, double length) const
+std::vector<double> UniformizationSubstitutionCount::getNumberOfSubstitutionsForEachType(size_t initialState, size_t finalState, double length) const
 {
   if (length < 0)
     throw Exception("UniformizationSubstitutionCount::getNumbersOfSubstitutions. Negative branch length: " + TextTools::toString(length) + ".");
@@ -226,7 +235,7 @@ void UniformizationSubstitutionCount::setSubstitutionModel(const SubstitutionMod
     initBMatrices_();
   }
   fillBMatrices_();
-	
+  
   miu_ = 0;
   for (unsigned int i = 0; i < nbStates_; ++i) {
     double diagQ = abs(model_->Qij(i, i));
@@ -234,6 +243,9 @@ void UniformizationSubstitutionCount::setSubstitutionModel(const SubstitutionMod
       miu_ = diagQ;
   }
 
+  if (miu_>10000)
+    throw Exception("UniformizationSubstitutionCount::setSubstitutionModel The maximum diagonal values of generator is above 10000. Abort, chose another mapping method.");
+
   //Recompute counts:
   computeCounts_(currentLength_);
 }
@@ -262,7 +274,8 @@ void UniformizationSubstitutionCount::weightsHaveChanged() throw (Exception)
   if (weights_->getAlphabet()->getAlphabetType() != register_->getAlphabet()->getAlphabetType())
     throw Exception("UniformizationSubstitutionCount::weightsHaveChanged. Incorrect alphabet type.");
 
-  fillBMatrices_();
+  //jdutheil on 25/07/14: not necessary if weights are only accounted for in the end.
+  //fillBMatrices_();
   
   //Recompute counts:
   if (currentLength_ > 0)
