@@ -103,12 +103,12 @@ class UniformizationSubstitutionCount:
 		
     UniformizationSubstitutionCount* clone() const { return new UniformizationSubstitutionCount(*this); }
 
-	public:
-		double getNumberOfSubstitutions(int initialState, int finalState, double length, size_t type = 1) const;
+public:
+    double getNumberOfSubstitutions(size_t initialState, size_t finalState, double length, size_t type = 1) const;
 
     Matrix<double>* getAllNumbersOfSubstitutions(double length, size_t type = 1) const;
     
-    std::vector<double> getNumberOfSubstitutionsForEachType(int initialState, int finalState, double length) const;
+    std::vector<double> getNumberOfSubstitutionsForEachType(size_t initialState, size_t finalState, double length) const;
    
     void setSubstitutionModel(const SubstitutionModel* model);
 
