@@ -59,15 +59,17 @@ using namespace std;
 
 /******************************************************************************/
 
+//TODO: jdutheil on 24/09/14: we should define a class "vector of models", insuring they share the same alphabet and StateMap for instance.
+//This is the only way to avoid a segfault in case the user provides a vector of models of length 0.
 AbstractWordSubstitutionModel::AbstractWordSubstitutionModel(
   const std::vector<SubstitutionModel*>& modelVector,
-  const std::string& st) :
-  AbstractParameterAliasable(st),
-  AbstractSubstitutionModel(AbstractWordSubstitutionModel::extractAlph(modelVector), st),
+  const std::string& prefix) :
+  AbstractParameterAliasable(prefix),
+  AbstractSubstitutionModel(AbstractWordSubstitutionModel::extractAlph(modelVector), modelVector[0]->getStateMap().clone(), prefix),
   new_alphabet_ (true),
   VSubMod_      (),
   VnestedPrefix_(),
-  Vrate_         (modelVector.size())
+  Vrate_        (modelVector.size())
 {
   enableEigenDecomposition(false);
   size_t i, j;
@@ -99,7 +101,7 @@ AbstractWordSubstitutionModel::AbstractWordSubstitutionModel(
     {
       VSubMod_.push_back(modelVector[i]);
       VnestedPrefix_.push_back(modelVector[i]->getNamespace());
-      VSubMod_[i]->setNamespace(st + TextTools::toString(i + 1) + "_" + VnestedPrefix_[i]);
+      VSubMod_[i]->setNamespace(prefix + TextTools::toString(i + 1) + "_" + VnestedPrefix_[i]);
       addParameters_(VSubMod_[i]->getParameters());
     }
   }
@@ -112,7 +114,7 @@ AbstractWordSubstitutionModel::AbstractWordSubstitutionModel(
       VnestedPrefix_.push_back(modelVector[0]->getNamespace());
       t += TextTools::toString(i + 1);
     }
-    VSubMod_[0]->setNamespace(st + t + "_" + VnestedPrefix_[0]);
+    VSubMod_[0]->setNamespace(prefix + t + "_" + VnestedPrefix_[0]);
     addParameters_(VSubMod_[0]->getParameters());
   }
 
@@ -124,9 +126,10 @@ AbstractWordSubstitutionModel::AbstractWordSubstitutionModel(
 
 AbstractWordSubstitutionModel::AbstractWordSubstitutionModel(
   const Alphabet* alph,
-  const std::string& st) :
-  AbstractParameterAliasable(st),
-  AbstractSubstitutionModel(alph, st),
+  StateMap* stateMap,
+  const std::string& prefix) :
+  AbstractParameterAliasable(prefix),
+  AbstractSubstitutionModel(alph, stateMap, prefix),
   new_alphabet_ (false),
   VSubMod_      (),
   VnestedPrefix_(),
@@ -138,9 +141,9 @@ AbstractWordSubstitutionModel::AbstractWordSubstitutionModel(
 AbstractWordSubstitutionModel::AbstractWordSubstitutionModel(
   SubstitutionModel* pmodel,
   unsigned int num,
-  const std::string& st) :
-  AbstractParameterAliasable(st),
-  AbstractSubstitutionModel(new WordAlphabet(pmodel->getAlphabet(), num), st),
+  const std::string& prefix) :
+  AbstractParameterAliasable(prefix),
+  AbstractSubstitutionModel(new WordAlphabet(pmodel->getAlphabet(), num), pmodel->getStateMap().clone(), prefix),
   new_alphabet_ (true),
   VSubMod_      (),
   VnestedPrefix_(),
@@ -158,7 +161,7 @@ AbstractWordSubstitutionModel::AbstractWordSubstitutionModel(
     t += TextTools::toString(i + 1);
   }
 
-  pmodel->setNamespace(st + t + "_" + VnestedPrefix_[0]);
+  pmodel->setNamespace(prefix + t + "_" + VnestedPrefix_[0]);
   addParameters_(pmodel->getParameters());
 }
 
@@ -377,7 +380,7 @@ void AbstractWordSubstitutionModel::updateMatrices()
 
     if (nbStop != 0)
     {
-      int gi = 0, gj = 0;
+      size_t gi = 0, gj = 0;
 
       RowMatrix<double> gk;
 
@@ -462,23 +465,18 @@ void AbstractWordSubstitutionModel::updateMatrices()
           isDiagonalizable_ = false;
       }
 
-
       // is it singular?
 
       // looking for the 0 eigenvector for which the non-stop right
       // eigen vector elements are equal.
       //
-      // there is a tolerance for numerical problems
-      //
 
-      size_t nulleigen = 0;
-      double val;
-      double seuil = 1;
-
-      isNonSingular_ = false;
-      while (!isNonSingular_)
+      if (isDiagonalizable_)
       {
-        nulleigen = 0;
+        size_t nulleigen = 0;
+        double val;
+
+        isNonSingular_ = false;
         while (nulleigen < salph - nbStop)
         {
           if ((abs(eigenValues_[nulleigen]) < NumConstants::SMALL()) && (abs(iEigenValues_[nulleigen]) < NumConstants::SMALL()))
@@ -486,19 +484,19 @@ void AbstractWordSubstitutionModel::updateMatrices()
             i = 0;
             while (vnull[i])
               i++;
-
+            
             val = rightEigenVectors_(i, nulleigen);
             i++;
             while (i < salph)
             {
               if (!vnull[i])
               {
-                if (abs(rightEigenVectors_(i, nulleigen) - val) > seuil * NumConstants::SMALL())
+                if (abs(rightEigenVectors_(i, nulleigen) - val) > NumConstants::SMALL())
                   break;
               }
               i++;
             }
-
+            
             if (i < salph)
               nulleigen++;
             else
@@ -510,44 +508,31 @@ void AbstractWordSubstitutionModel::updateMatrices()
           else
             nulleigen++;
         }
-        if (seuil > 100)
-        {
-          ApplicationTools::displayWarning("!!! Equilibrium frequency of the model " + getName() + " has a precision less than " ""  + TextTools::toString(seuil * NumConstants::SMALL()) + ". There may be some computing issues.");
-          ApplicationTools::displayWarning("!!! Taylor series used instead");
-          break;
-        }
-        else
-          seuil *= 10;
-      }
-
-      if (isNonSingular_)
-      {
-        eigenValues_[nulleigen] = 0; // to avoid approximation errors on long long branches
-        iEigenValues_[nulleigen] = 0; // to avoid approximation errors on long long branches
-
-        for (i = 0; i < salph; i++)
-        {
-          freq_[i] = leftEigenVectors_(nulleigen, i);
-        }
-
-        x = 0;
-        for (i = 0; i < salph; i++)
-        {
-          x += freq_[i];
-        }
-
-        for (i = 0; i < salph; i++)
+        
+        if (isNonSingular_)
         {
+          eigenValues_[nulleigen] = 0; // to avoid approximation errors on long long branches
+          iEigenValues_[nulleigen] = 0; // to avoid approximation errors on long long branches
+          
+          for (i = 0; i < salph; i++)
+            freq_[i] = leftEigenVectors_(nulleigen, i);
+          
+          x = 0;
+          for (i = 0; i < salph; i++)
+            x += freq_[i];
+          
+          for (i = 0; i < salph; i++)
           freq_[i] /= x;
         }
-      }
-      else
-      {
-        ApplicationTools::displayMessage("Unable to find eigenvector for eigenvalue 1. Taylor series used instead.");
-        isDiagonalizable_ = false;
+      
+        else
+        {
+          ApplicationTools::displayMessage("Unable to find eigenvector for eigenvalue 1. Taylor series used instead.");
+          isDiagonalizable_ = false;
+        }
       }
     }
-
+    
     // if rightEigenVectors_ is singular
     catch (ZeroDivisionException& e)
     {
@@ -586,9 +571,7 @@ void AbstractWordSubstitutionModel::updateMatrices()
 
     x = 0;
     for (i = 0; i < salph; i++)
-    {
       x += freq_[i] * generator_(i, i);
-    }
 
     MatrixTools::scale(generator_, -1. / x);
     for (i = 0; i < salph; i++)
@@ -604,12 +587,9 @@ void AbstractWordSubstitutionModel::updateMatrices()
   // compute the exchangeability_
 
   for (i = 0; i < size_; i++)
-  {
     for (j = 0; j < size_; j++)
-    {
       exchangeability_(i, j) = generator_(i, j) / freq_[j];
-    }
-  }
+
 }
 
 void AbstractWordSubstitutionModel::setFreq(std::map<int, double>& freqs)
