@@ -52,8 +52,8 @@ using namespace std;
 
 BinarySubstitutionModel::BinarySubstitutionModel(const BinaryAlphabet* alpha, double kappa) :
   AbstractParameterAliasable("Binary."),
-  AbstractSubstitutionModel(alpha, "Binary."),
-  AbstractReversibleSubstitutionModel(alpha, "Binary."),
+  //AbstractSubstitutionModel(alpha, new CanonicalStateMap(alpha, false), "Binary."),
+  AbstractReversibleSubstitutionModel(alpha, new CanonicalStateMap(alpha, false), "Binary."),
   kappa_(kappa),
   lambda_(0),
   exp_(0),
