@@ -61,8 +61,7 @@ F84::F84(
     double piG,
     double piT) :
   AbstractParameterAliasable("F84."),
-  AbstractSubstitutionModel(alpha, "F84."),
-  AbstractReversibleSubstitutionModel(alpha, "F84."),
+  AbstractReversibleSubstitutionModel(alpha, new CanonicalStateMap(alpha, false), "F84."),
   kappa_(kappa), piA_(piA), piC_(piC), piG_(piG), piT_(piT), piY_(), piR_(),
   r_(), k1_(), k2_(), theta_(piG + piC), theta1_(piA / (1. - theta_)), theta2_(piG / theta_),
   l_(), exp1_(), exp2_(), p_(size_, size_)
@@ -192,7 +191,7 @@ void F84::updateMatrices()
   
 /******************************************************************************/
 
-double F84::Pij_t(int i, int j, double d) const
+double F84::Pij_t(size_t i, size_t j, double d) const
 {
   l_ = rate_ * r_ * d;
   exp1_ = exp(-k1_*l_);
@@ -241,7 +240,7 @@ double F84::Pij_t(int i, int j, double d) const
 
 /******************************************************************************/
 
-double F84::dPij_dt(int i, int j, double d) const
+double F84::dPij_dt(size_t i, size_t j, double d) const
 {
   l_ = rate_ * r_ * d;
   exp1_ = exp(-k1_*l_);
@@ -290,7 +289,7 @@ double F84::dPij_dt(int i, int j, double d) const
 
 /******************************************************************************/
 
-double F84::d2Pij_dt2(int i, int j, double d) const
+double F84::d2Pij_dt2(size_t i, size_t j, double d) const
 {
   double r_2 = rate_ * rate_ * r_ * r_;
   l_ = rate_ * r_ * d;
