@@ -58,7 +58,7 @@ RN95s::RN95s(const NucleicAlphabet* alphabet,
              double gamma,
              double delta) :
   AbstractParameterAliasable("RN95s."),
-  AbstractSubstitutionModel(alphabet, "RN95s."),
+  AbstractSubstitutionModel(alphabet, new CanonicalStateMap(alphabet, false), "RN95s."),
   alpha_(),
   beta_(),
   gamma_(),
@@ -178,35 +178,37 @@ void RN95s::updateMatrices()
 
 // Need formula
 
-  try
+  if (enableEigenDecomposition())
   {
-    MatrixTools::inv(rightEigenVectors_, leftEigenVectors_);
-    isNonSingular_ = true;
-    isDiagonalizable_ = true;
-    for (unsigned int i = 0; i < size_ && isDiagonalizable_; i++)
+    try
     {
-      if (abs(iEigenValues_[i]) > NumConstants::TINY())
-        isDiagonalizable_ = false;
+      MatrixTools::inv(rightEigenVectors_, leftEigenVectors_);
+      isNonSingular_ = true;
+      isDiagonalizable_ = true;
+      for (unsigned int i = 0; i < size_ && isDiagonalizable_; i++)
+      {
+        if (abs(iEigenValues_[i]) > NumConstants::TINY())
+          isDiagonalizable_ = false;
+      }
     }
+    catch (ZeroDivisionException& e)
+    {
+      ApplicationTools::displayMessage("Singularity during  diagonalization. Taylor series used instead.");
+      
+      isNonSingular_ = false;
+      isDiagonalizable_ = false;
+      MatrixTools::Taylor(generator_, 30, vPowGen_);
+    }
+    
+    // and the exchangeability_
+    for (size_t i = 0; i < size_; i++)
+      for (size_t j = 0; j < size_; j++)
+        exchangeability_(i,j) = generator_(i,j) / freq_[j];
   }
-  catch (ZeroDivisionException& e)
-  {
-    ApplicationTools::displayMessage("Singularity during  diagonalization. Taylor series used instead.");
-
-    isNonSingular_ = false;
-    isDiagonalizable_ = false;
-    MatrixTools::Taylor(generator_, 30, vPowGen_);
-  }
-
-  // and the exchangeability_
-  for (size_t i = 0; i < size_; i++)
-    for (size_t j = 0; j < size_; j++)
-      exchangeability_(i,j) = generator_(i,j) / freq_[j];
-
 }
 
 /******************************************************************************/
-double RN95s::Pij_t(int i, int j, double d) const
+double RN95s::Pij_t(size_t i, size_t j, double d) const
 {
   l_ = rate_ * r_ * d;
   exp1_ = exp(-l_);
@@ -261,7 +263,7 @@ double RN95s::Pij_t(int i, int j, double d) const
 }
 
 /******************************************************************************/
-double RN95s::dPij_dt(int i, int j, double d) const
+double RN95s::dPij_dt(size_t i, size_t j, double d) const
 {
   l_ = rate_ * r_ * d;
   exp1_ = -1.* rate_* r_* exp(-1. * l_);
@@ -316,7 +318,7 @@ double RN95s::dPij_dt(int i, int j, double d) const
 }
 
 /******************************************************************************/
-double RN95s::d2Pij_dt2(int i, int j, double d) const
+double RN95s::d2Pij_dt2(size_t i, size_t j, double d) const
 {
   l_ = rate_ * r_ * d;
   exp1_ = 1. * rate_ * r_ * 1.* rate_* r_* exp(-1. * l_);
