@@ -58,12 +58,10 @@ using namespace std;
 
 WordSubstitutionModel::WordSubstitutionModel(
   const std::vector<SubstitutionModel*>& modelVector,
-  const std::string& st) :
-  AbstractParameterAliasable((st == "") ? "Word." : st),
-  AbstractSubstitutionModel(AbstractWordSubstitutionModel::extractAlph(modelVector),
-                            (st == "") ? "Word." : st),
+  const std::string& prefix) :
+  AbstractParameterAliasable((prefix == "") ? "Word." : prefix),
   AbstractWordSubstitutionModel(modelVector,
-                                (st == "") ? "Word." : st)
+                                (prefix == "") ? "Word." : prefix)
 {
   size_t i, nbmod = VSubMod_.size();
 
@@ -78,24 +76,20 @@ WordSubstitutionModel::WordSubstitutionModel(
 
 WordSubstitutionModel::WordSubstitutionModel(
   const Alphabet* alph,
-  const std::string& st) :
-  AbstractParameterAliasable((st == "") ? "Word." : st),
-  AbstractSubstitutionModel(alph,
-                            (st == "") ? "Word." : st),
-  AbstractWordSubstitutionModel(alph,
-                                (st == "") ? "Word." : st)
+  StateMap* stateMap,
+  const std::string& prefix) :
+  AbstractParameterAliasable((prefix == "") ? "Word." : prefix),
+  AbstractWordSubstitutionModel(alph, stateMap, (prefix == "") ? "Word." : prefix)
 {}
 
 WordSubstitutionModel::WordSubstitutionModel(
   SubstitutionModel* pmodel,
   unsigned int num,
-  const std::string& st) :
-  AbstractParameterAliasable((st == "") ? "Word." : st),
-  AbstractSubstitutionModel(pmodel->getAlphabet(),
-                            (st == "") ? "Word." : st),
+  const std::string& prefix) :
+  AbstractParameterAliasable((prefix == "") ? "Word." : prefix),
   AbstractWordSubstitutionModel(pmodel,
                                 num,
-                                (st == "") ? "Word." : st)
+                                (prefix == "") ? "Word." : prefix)
 {
   size_t i;
 
@@ -137,10 +131,10 @@ void WordSubstitutionModel::completeMatrices()
   {
     freq_[i] = 1;
     j = i;
-    for (p = nbmod - 1; p >= 0; p--)
+    for (p = nbmod; p > 0; p--)
     {
-      m = VSubMod_[p]->getNumberOfStates();
-      freq_[i] *= VSubMod_[p]->getFrequencies()[j % m];
+      m = VSubMod_[p-1]->getNumberOfStates();
+      freq_[i] *= VSubMod_[p-1]->getFrequencies()[j % m];
       j /= m;
     }
   }
