@@ -5,7 +5,7 @@
 //
 
 /*
-   Copyright or © or Copr. CNRS, (November 16, 2004)
+   Copyright or © or Copr. Bio++ Development Team, (November 16, 2004)
 
    This software is a computer program whose purpose is to provide classes
    for phylogenetic data analysis.
@@ -55,7 +55,7 @@
 #include <Bpp/Numeric/Prob/ConstantDistribution.h>
 #include <Bpp/Numeric/Matrix/MatrixTools.h>
 
-// From SeqLib:
+// From bpp-seq:
 #include <Bpp/Seq/Alphabet/DNA.h>
 #include <Bpp/Seq/Container/VectorSiteContainer.h>
 
@@ -718,10 +718,11 @@ DistanceMatrix* TreeTools::getDistanceMatrix(const Tree& tree)
 
 void TreeTools::midpointRooting(Tree& tree)
 {
+  throw Exception("TreeTools::midpointRooting(Tree). This function is deprecated, use TreeTemplateTools::midRoot instead!");
   if (tree.isRooted())
     tree.unroot();
   DistanceMatrix* dist = getDistanceMatrix(tree);
-  vector<size_t> pos = MatrixTools::whichMax(*dist);
+  vector<size_t> pos = MatrixTools::whichMax(dist->asMatrix());
   double dmid = (*dist)(pos[0], pos[1]) / 2;
   int id1 = tree.getLeafId(dist->getName(pos[0]));
   int id2 = tree.getLeafId(dist->getName(pos[1]));
@@ -817,6 +818,26 @@ VectorSiteContainer* TreeTools::MRPEncode(const vector<Tree*>& vecTr)
 
 /******************************************************************************/
 
+VectorSiteContainer* TreeTools::MRPEncodeMultilabel(const vector<Tree*>& vecTr)
+{
+    vector<BipartitionList*> vecBipL;
+    for (size_t i = 0; i < vecTr.size(); i++)
+    {
+        vecBipL.push_back(new BipartitionList(*vecTr[i]));
+    }
+    
+    VectorSiteContainer* cont = BipartitionTools::MRPEncodeMultilabel(vecBipL);
+    
+    for (size_t i = 0; i < vecTr.size(); i++)
+    {
+        delete vecBipL[i];
+    }
+    
+    return cont;
+}
+
+/******************************************************************************/
+
 bool TreeTools::haveSameTopology(const Tree& tr1, const Tree& tr2)
 {
   size_t jj, nbbip;
@@ -981,7 +1002,7 @@ BipartitionList* TreeTools::bipartitionOccurrences(const vector<Tree*>& vecTr, v
   {
     if (bipScore[i - 1] == 0)
     {
-      bipScore.erase(bipScore.begin() + i - 1);
+      bipScore.erase(bipScore.begin() + static_cast<ptrdiff_t>(i - 1));
       mergedBipL->deleteBipartition(i - 1);
     }
   }
@@ -1100,7 +1121,7 @@ Tree* TreeTools::MRP(const vector<Tree*>& vecTr)
 
 /******************************************************************************/
 
-void TreeTools::computeBootstrapValues(Tree& tree, const vector<Tree*>& vecTr, bool verbose)
+void TreeTools::computeBootstrapValues(Tree& tree, const vector<Tree*>& vecTr, bool verbose, int format)
 {
   vector<int> index;
   BipartitionList bpTree(tree, true, &index);
@@ -1117,7 +1138,7 @@ void TreeTools::computeBootstrapValues(Tree& tree, const vector<Tree*>& vecTr, b
     {
       if (BipartitionTools::areIdentical(bpTree, i, *bpList, j))
       {
-        bootstrapValues[i] = (double) occurences[j] * 100. / (double) vecTr.size();
+        bootstrapValues[i] = format >= 0 ? round(static_cast<double>(occurences[j]) * pow(10., 2 + format) / static_cast<double>(vecTr.size())) / pow(10., format) : static_cast<double>(occurences[j]);
         break;
       }
     }
@@ -1261,3 +1282,33 @@ TreeTools::Moments_ TreeTools::statFromNode_(Tree& tree, int rootId)
   return m;
 }
 
+/******************************************************************************/
+
+Tree* TreeTools::MRPMultilabel(const vector<Tree*>& vecTr)
+{
+    // matrix representation
+    VectorSiteContainer* sites = TreeTools::MRPEncode(vecTr);
+    
+    // starting bioNJ tree
+    const DNA* alphabet = dynamic_cast<const DNA*>(sites->getAlphabet());
+    JCnuc* jc = new JCnuc(alphabet);
+    ConstantDistribution* constRate = new ConstantDistribution(1.);
+    DistanceEstimation distFunc(jc, constRate, sites, 0, true);
+    BioNJ bionjTreeBuilder(false, false);
+    bionjTreeBuilder.setDistanceMatrix(*(distFunc.getMatrix()));
+    bionjTreeBuilder.computeTree();
+    if (ApplicationTools::message)
+        ApplicationTools::message->endLine();
+    TreeTemplate<Node>* startTree = new TreeTemplate<Node>(*bionjTreeBuilder.getTree());
+    
+    // MP optimization
+    DRTreeParsimonyScore* MPScore = new DRTreeParsimonyScore(*startTree, *sites, false);
+    MPScore = OptimizationTools::optimizeTreeNNI(MPScore, 0);
+    delete startTree;
+    Tree* retTree = new TreeTemplate<Node>(MPScore->getTree());
+    delete MPScore;
+    
+    return retTree;
+}
+
+
