@@ -413,6 +413,7 @@ class TreeTools
      * The root is then set on the branch located at half this distance.
      *
      * @param tree The tree to (re)root.
+     * @deprecated Use TreeTemplateTools::midRoot instead!
      */
     static void midpointRooting(Tree& tree);
     /** @} */
@@ -562,11 +563,22 @@ class TreeTools
      */
     static VectorSiteContainer* MRPEncode(const std::vector<Tree*>& vecTr);
 
+    /**
+     * @brief Creates a sequence data set corresponding to the Matrix Representation of the input multilabel trees
+     *
+     * @author Nicolas Galtier and Bastien Boussau
+     * Trees can have distinct sets of elements - missing data will be represented as 'N'.
+     * The output alignment (DNA sequences including only A, C and N)) is ready for maximum parsimony analysis
+     * according to the MRP supertree method.
+     */
+    static VectorSiteContainer* MRPEncodeMultilabel(const std::vector<Tree*>& vecTr);
+
     /**
      * @brief Tells whether two trees have the same unrooted topology
      *
-     * @author Nicolas Galtier
      * Note that the location of the root, if any, is ignored.
+     *
+     * @author Nicolas Galtier
      */
     static bool haveSameTopology(const Tree& tr1, const Tree& tr2);
 
@@ -667,11 +679,13 @@ class TreeTools
     /**
      * @brief Compute bootstrap values.
      *
-     * @param tree Input tree. the BOOTSTRAP banch property of the tree will be modified if it already exists.
-     * @param vecTr A list of trees to compare to 'tree'.
+     * @param tree    Input tree. the BOOTSTRAP banch property of the tree will be modified if it already exists.
+     * @param vecTr   A list of trees to compare to 'tree'.
      * @param verbose Tell if a progress bar should be displayed.
+     * @param format  If null or positive, bootstrap values are reported as percentage, with the given number of decimal digits.
+     *                If negative, bootstrap calues are the raw number of tree occurrences.
      */
-    static void computeBootstrapValues(Tree& tree, const std::vector<Tree*>& vecTr, bool verbose = true);
+    static void computeBootstrapValues(Tree& tree, const std::vector<Tree*>& vecTr, bool verbose = true, int format = 0);
 	
     /**
      * @brief Determine the mid-point position of the root along the branch that already contains the root. Consequently, the topology of the rooted tree remains identical.
@@ -682,6 +696,20 @@ class TreeTools
      * @param tree The rooted tree for which the root has to be moved to its mid-point position, along the branch where it already stands.
      */    
     static void constrainedMidPointRooting(Tree& tree);
+    
+    /**
+     * @brief Matrix Representation Parsimony supertree method for multilabel trees
+     *
+     * This implementation of the MRP method takes a BIONJ tree (Jukes-Cantor distances)
+     * as the starting tree and optimizes the parsimony score using only NNI (in a
+     * PHYML-like way).
+     *
+     * @author Nicolas Galtier slightly modified by Bastien Boussau
+     * @param vecTr A vector of trees.
+     * @return The MRP super tree.
+     */
+    static Tree* MRPMultilabel(const std::vector<Tree*>& vecTr);
+
 	
     /**
      * @name Some properties.
